#
# agentmaneg.rb
#
#   Copyright (c) 2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'amstd/must'
require 'amstd/extmod'


module EST

  class AgentManager

def dprint
  puts "---- #{type} ----"
  puts '---- klass'
  @obj.each {|name,i| puts "#{name}: #{i}" }
  puts '---- object'
  @obj.each {|name,i| puts "#{name}: #{i.type}" }
end
    
    def initialize
      @klass = {}
      @obj = {}
      init_after
    end

    def init_after
    end
    private :init_after

    def register_class( name, klass )
      klass.must Class
      @klass[ name ] = klass
    end

def parg( name, rc )
  puts "name  ='#{name}'"
  puts "rc    =#{rc.inspect}"
  puts "@klass=#{@klass.inspect}"
end
    
    def get_class( name, rc )
      c = nil
begin
      c =
      (rc['type'] && @klass[ rc['type'] ]) ||
      (name && @klass[name]) ||
      @klass['default']
ensure
  unless c
    parg name, rc
    bug!
  end
end
      c
    end

    def createobj( name, rc, *args )
begin
      c = get_class( name, rc )
      c.new( rc, *args )
rescue ArgumentError
  p c
  raise
end
    end
    private :createobj

    def create( rc, *args )
      createobj nil, rc, *args
    end
    private :create

    def add( name, rc, *args )
      @obj[ name ] = createobj( name, rc, *args )
    end

    def get( name )
      @obj[ name ]
    end

  end


  class SenderManager < AgentManager

    def init_after
      register_class 'default', SMTPclient
      register_class 'smtp', SMTPclient
    end

    def select( name )
      @current = get( name )
    end

    attr :current

  end


  class PopperManager < AgentManager
  
    def init_after
      register_class 'default', POPclient
      register_class 'pop', POPclient
      register_class 'pop3', POPclient
      register_class 'apop', APOPclient
    end
    private :init_after

    def select( *names )
      @current = names.collect {|n| get( n ) }
    end

    attr :current

  end


  class ComposerManager < AgentManager

    def init_after
      register_class 'new-mail', NewMailComposer
      register_class 'reply-mail', ReplyMailComposer
      register_class 'forward-mail', ForwardMailComposer
    end
    private :init_after
  
  end


  class GetterManager < AgentManager

    def init_after
      register_class 'seen',      SeenGetter
      register_class 'multipart', MultipartGetter
      register_class 'date',      DateGetter
      register_class 'from',      FromGetter
      register_class 'subject',   SubjectGetter
    end
    private :init_after

    public :create

  end


  class RefilerManager < AgentManager

    def init_after
      register_class 'default', Refiler
    end

  end

  
  class ProfileManager < AgentManager

    def init_after
    end
    private :init_after
  
    def add( name, rc )
      @obj[ name ] = rc
    end

    def select( to )
      unless val = get( to ) then
        CORE.warn "EST Warning: profile '#{to}' not exist: command canceled."
        return
      end

=begin c
      unless ent = val['mailbox'][to] then
        CORE.warn <<S
EST Warning: non-exist mbox profile '#{to}' was passed:
EST Warning: command canceled.
S
        return
      end
=end
      CORE.mailbox.inbox = val['mailbox']['inbox']
      CORE.mailbox.draft = val['mailbox']['draft']
      CORE.mailbox.trash = val['mailbox']['trash']

      CORE.sender.select val['smtp-server']
      CORE.popper.select val['pop-server']
      # CORE.composer.select val['composer']
      @current = val
    end

    alias profile= select

    attr :current


    def get_current( name )
      arr = name.split('/')
      ent = current
      arr.each do |i|
        ent = ent[i]
      end
      mana = CORE.send(arr[0])
      mana.get( ent )
    end


    def confirm_config( profnm, profiles )
      ent = profiles[ profnm ]
      unless ent['mailbox'] and
             ent['smtp-server'] and
             ent['pop-server'] and
             ent['composer']
        CORE.warn "EST Warning: profile #{profnm} not exist: delete entry"
        profiles.delete profnm
      end
    end

  end

end
